.pln{
    color:#000
}

@media screen{
    .str{
        color:#BA2121;
    }.kwd{
        color:#008000;
        font-weight: bold;
    }.com{
        color:#408080;
        font-style: italic;
    }.typ{
        color:#606
    }.lit{
        color:#666666;
    }.pun,.opn,.clo{
    }.tag{
        color:#008
    }.atn{
        color:#606
    }.atv{
        color:#080
    }.dec,.var{
        color:#606
    }.fun{
        color:red
    }
}

pre.prettyprint{
    padding:2px;
    border:1px solid #888
}

ol.linenums{
    margin-top:0;
    margin-bottom:0
}

li.L0,li.L1,li.L2,li.L3,li.L5,li.L6,li.L7,li.L8{
    list-style-type:none
}

li.L1,li.L3,li.L5,li.L7,li.L9{
    background:#eee
}