<!DOCTYPE HTML>
<html>

<head>
    <meta charset="utf-8">

    <title>{% block title %}IPython Notebook{% end %}</title>

    <link rel="stylesheet" href="{{static_url("jquery/css/themes/base/jquery-ui.min.css") }}" type="text/css" />
    <link rel="stylesheet" href="{{static_url("css/boilerplate.css") }}" type="text/css" />
    <link rel="stylesheet" href="{{static_url("css/fbm.css") }}" type="text/css" />
    <link rel="stylesheet" href="{{static_url("css/page.css") }}" type="text/css"/>
    {% block stylesheet %}
    {% end %}

    {% block meta %}
    {% end %}

</head>

<body {% block params %}{% end %}>

<div id="header">
    <span id="ipython_notebook"><h1><a href={{base_project_url}} alt='dashboard'><img src='{{static_url("ipynblogo.png") }}' alt='IPython Notebook'/></a></h1></span>

    {% block login_widget %}

      <span id="login_widget">
        {% if logged_in %}
          <button id="logout">Logout</button>
        {% elif login_available and not logged_in %}
          <button id="login">Login</button>
        {% end %}
      </span>

    {% end %}

    {% block header %}
    {% end %}
</div>

<div id="site">
{% block site %}
{% end %}
</div>

<script src="{{static_url("jquery/js/jquery-1.7.1.min.js") }}" type="text/javascript" charset="utf-8"></script>
<script src="{{static_url("jquery/js/jquery-ui.min.js") }}" type="text/javascript" charset="utf-8"></script>
<script src="{{static_url("js/namespace.js") }}" type="text/javascript" charset="utf-8"></script>
<script src="{{static_url("js/page.js") }}" type="text/javascript" charset="utf-8"></script>
<script src="{{static_url("js/loginwidget.js") }}" type="text/javascript" charset="utf-8"></script>

{% block script %}
{% end %}

</body>

</html>
